@@ -1,26 +1,69 @@
-#' Fit a generalized additive latent and mixed model
+#' @title Fit a generalized additive latent and mixed model
 #'
-#' This function fits a generalized additive latent and mixed model (GALAMMs),
-#' as described in
-#' \insertCite{sorensenLongitudinalModelingAgeDependent2023;textual}{galamm}.
-#' The building blocks of these models are generalized additive mixed models
-#' (GAMMs) \insertCite{woodGeneralizedAdditiveModels2017a}{galamm}, of which
-#' generalized linear mixed models
-#' \insertCite{breslowApproximateInferenceGeneralized1993,harvilleMaximumLikelihoodApproaches1977,hendersonBestLinearUnbiased1975,lairdRandomEffectsModelsLongitudinal1982}{galamm}
-#' are special cases. GALAMMs extend upon GAMMs by allowing factor structures,
-#' as commonly used to model hypothesized latent traits underlying observed
-#' measurements. In this sense, GALAMMs are an extension of generalized linear
-#' latent and mixed models (GLLAMMs)
-#' \insertCite{skrondalGeneralizedLatentVariable2004,rabe-heskethGeneralizedMultilevelStructural2004}{galamm}
-#' which allows semiparametric estimation. The implemented algorithm used to
-#' compute model estimates is described in
-#' \insertCite{sorensenLongitudinalModelingAgeDependent2023;textual}{galamm},
-#' and is an extension of the algorithm used for fitting generalized linear
-#' mixed models by the \code{lme4} package
-#' \insertCite{batesFittingLinearMixedEffects2015}{galamm}. The syntax used to
-#' define factor structures is based on that used by the \code{PLmixed} package,
-#' which is detailed in
-#' \insertCite{rockwoodEstimatingComplexMeasurement2019;textual}{galamm}.
+#' @description This function fits a generalized additive latent and mixed model
+#'   (GALAMMs), as described in
+#'   \insertCite{sorensenLongitudinalModelingAgeDependent2023;textual}{galamm}.
+#'   The building blocks of these models are generalized additive mixed models
+#'   (GAMMs) \insertCite{woodGeneralizedAdditiveModels2017a}{galamm}, of which
+#'   generalized linear mixed models
+#'   \insertCite{breslowApproximateInferenceGeneralized1993,harvilleMaximumLikelihoodApproaches1977,hendersonBestLinearUnbiased1975,lairdRandomEffectsModelsLongitudinal1982}{galamm}
+#'   are special cases. GALAMMs extend upon GAMMs by allowing factor structures,
+#'   as commonly used to model hypothesized latent traits underlying observed
+#'   measurements. In this sense, GALAMMs are an extension of generalized linear
+#'   latent and mixed models (GLLAMMs)
+#'   \insertCite{skrondalGeneralizedLatentVariable2004,rabe-heskethGeneralizedMultilevelStructural2004}{galamm}
+#'   which allows semiparametric estimation. The implemented algorithm used to
+#'   compute model estimates is described in
+#'   \insertCite{sorensenLongitudinalModelingAgeDependent2023;textual}{galamm},
+#'   and is an extension of the algorithm used for fitting generalized linear
+#'   mixed models by the \code{lme4} package
+#'   \insertCite{batesFittingLinearMixedEffects2015}{galamm}. The syntax used to
+#'   define factor structures is based on that used by the \code{PLmixed}
+#'   package, which is detailed in
+#'   \insertCite{rockwoodEstimatingComplexMeasurement2019;textual}{galamm}.
+#'
+#' @srrstats {G1.0} Primary references shown in the description.
+#' @srrstats {G1.3} Statistical terminology defined in detail in the references.
+#' @srrstats {G1.4} Function documented with roxygen2.
+#' @srrstats {G2.0} Assertions on length of inputs are made in the beginning of
+#'   the galamm() function.
+#' @srrstats {G2.0a} Secondary documentation of expectations on lengths of
+#'   inputs provided for the parameters in the descriptions below. This applies
+#'   in particular to \code{lambda}, \code{factor}, \code{load.var}, and
+#'   \code{factor_interactions}, as well as \code{family} and
+#'   \code{family_mapping}.
+#' @srrstats {G2.1} Assertions on types of input implemented in galamm function.
+#' @srrstats {G2.1a} Documentation on expected data types provided for all
+#'   inputs in the documentation below.
+#' @srrstats {G2.2} Assertions on the lengths of arguments are implemented in
+#'   galamm.
+#' @srrstats {G2.3,G2.3a} match.arg() used on "na.action" argument.
+#' @srrstats {G2.3,G2.3b} Arguments "family", "load.var", "factor", and the elements
+#'   of the "start" argument are case sensitive. This is stated in the
+#'   documentation below.
+#' @srrstats {G2.4,G2.4a} Internally, objects family_mapping, weights_mapping and
+#'   lambda_mapping_X are explicitly converted to integer using as.integer().
+#' @srrstats {G2.4,G2.4b} as.numeric() used multiple places throughout the code for
+#'   explicitly converting to continuous.
+#' @srrstats {G2.6} If \code{lambda} is provided as a vector, it will be
+#'   converted to a matrix with a single column, and a message will be printed.
+#' @srrstats {G2.7} Both \code{tibble}s and \code{data.table}s are accepted in
+#'   the \code{data} argument.
+#' @srrstats {G2.10} \code{drop = FALSE} is used regularly in the code, when
+#'   extract columns from \code{data.frame}s or \code{matrix}.
+#' @srrstats {G2.13} Checks for missing data implemented in the preprocessing
+#'   steps of galamm. Note that in the argument \code{lambda}, \code{NA} values
+#'   mean that the matrix element is an unknown parameter.
+#' @srrstats {G2.14} Users can set options for handling missing values through
+#'   the argument \code{na.action}.
+#' @srrstats {G2.14a} Users can set \code{na.action = "na.fail"}.
+#' @srrstats {G2.15} If \code{na.action = "na.omit"} or \code{na.action =
+#'   "na.exclude"}, missing values in \code{data} are explicitly removed.
+#'   Otherwise, if \code{na.action = "na.fail"}, missing values in \code{data}
+#'   cause an error. In any case, data with potential missingness are never
+#'   provided to any base R functions.
+#' @srrstats {G2.16} \code{NaN}, \code{Inf}, or \code{-Inf} in \code{data}
+#'   causes an error. The same happens with such values in \code{lambda}.
 #'
 #' @param formula A formula specifying the model. Smooth terms are defined in
 #'   the style of the \code{mgcv} and \code{gamm4} packages, see
@@ -40,49 +83,66 @@
 #' @param data A data.frame containing all the variables specified by the model
 #'   formula, with the exception of factor loadings.
 #'
-#' @param family A vector containing one or more model families. For each
-#'   element in \code{family} there should be a corresponding element in
-#'   \code{family_mapping} specifying which elements of the response are
-#'   conditionally distributed according to the given family. Currently family
-#'   can be one of \code{gaussian}, \code{binomial}, and \code{poisson}, and
-#'   only canonical link functions are supported.
+#' @param family A a list or character vector containing one or more model
+#'   families. For each element in \code{family} there should be a corresponding
+#'   element in \code{family_mapping} specifying which elements of the response
+#'   are conditionally distributed according to the given family. Currently
+#'   family can be one of \code{gaussian}, \code{binomial}, and \code{poisson},
+#'   and only canonical link functions are supported. The family arguments can
+#'   either be provided as character values, e.g., \code{c("gaussian",
+#'   "poisson")} or \code{list("gaussian", "poisson")}, as function names, e.g.,
+#'   \code{c(gaussian, poisson)} or \code{list(gaussian, poisson)}, or as
+#'   function calls, e.g., \code{list(gaussian(), poisson())}. In the latter
+#'   case, they must be provided in a list, and bot as a vector. Mixing the
+#'   different ways of describing the family also works, e.g.,
+#'   \code{list("gaussian", poisson())}, but in this case they must be provided
+#'   in a list. When provided as character values, the argument is case
+#'   sensitive.
 #'
-#' @param family_mapping Optional integer vector mapping from the elements of
-#'   \code{family} to rows of \code{data}. Defaults to \code{rep(1L,
+#'
+#' @param family_mapping Optional vector mapping from the elements of
+#'   \code{family} to rows of \code{data}. Defaults to \code{rep(1,
 #'   nrow(data))}, which means that all observations are distributed according
-#'   to the first element of \code{family}.
+#'   to the first element of \code{family}. The length of \code{family_mapping}
+#'   must be identical to the number of observations, \code{nrow(data)}.
 #'
 #' @param load.var Optional character specifying the name of the variable in
-#'   \code{data} identifying what the factors load onto. That is, each unique
-#'   value of \code{load.var} corresponds to a unique factor loading. Currently
-#'   only a single loading is supported, so \code{load.var} must have length
-#'   one. Default to \code{NULL}, which means that there are no loading
-#'   variables.
+#'   \code{data} identifying what the factors load onto. Default to \code{NULL},
+#'   which means that there are no loading variables. Argument is case
+#'   sensitive.
 #'
-#' @param lambda Optional list of factor loading matrices. Numerical values
-#'   indicate that the given value is fixed, while \code{NA} means that the
-#'   entry is a parameter to be estimated. Defaults to \code{NULL}, which means
-#'   that there are no factor loading matrices.
+#' @param lambda Optional factor loading matrix. Numerical values indicate that
+#'   the given value is fixed, while \code{NA} means that the entry is a
+#'   parameter to be estimated. Numerical values can only take the values 0 or
+#'   1. The number of columns of \code{lambda} must be identical to the number
+#'   of elements in \code{factor}. Defaults to \code{NULL}, which means that
+#'   there is no factor loading matrix. If \code{lambda} is provided as a
+#'   vector, it will be converted to a \code{matrix} with a single column.
 #'
-#' @param factor Optional list of character vectors identical to the factor
-#'   loadings specified in \code{formula}. For each list element, the \eqn{j}th
-#'   entry in the character vector corresponds to the \eqn{j}th column of the
-#'   corresponding matrix in \code{lambda}. Defaults to \code{NULL}, which means
-#'   that there are no factor loadings.
+#' @param factor Optional character vector whose \eqn{j}th entry corresponds to
+#'   the \eqn{j}th column of the corresponding matrix in \code{lambda}. The
+#'   number of elements in \code{factor} must be equal to the number of columns
+#'   in \code{lambda}. Defaults to \code{NULL}, which means that there are no
+#'   factor loadings. Argument is case sensitive.
 #'
-#' @param factor_interactions Optional list of length equal to the list provided
-#'   in the \code{factor} argument. If provided, each element of the lists
-#'   should itself be a list of length equal to the number of columns in the
-#'   corresponding matrix provided to the \code{lambda} argument. Each list
-#'   element should be a \code{formula} object containing the write-hand side of
-#'   a regression model, of the form \code{~ x + z}. Defaults to \code{NULL},
-#'   which means that no factor interactions are used.
+#' @param factor_interactions Optional list of length equal to the number of
+#'   columns in \code{lambda}. Each list element should be a \code{formula}
+#'   object containing the write-hand side of a regression model, of the form
+#'   \code{~ x + z}. Defaults to \code{NULL}, which means that no factor
+#'   interactions are used.
+#'
+#' @param na.action Character of length one specifying a function which
+#'   indicates what should happen when the data contains \code{NA}s. The
+#'   defaults is set to the \code{na.action} setting of \code{options}, which
+#'   can be seen with \code{options("na.action")}. The other alternatives are
+#'   \code{"na.fail"} or \code{"na.exclude"}, which means that the function
+#'   fails if there as \code{NA}s in \code{data}.
 #'
 #' @param start Optional named list of starting values for parameters. Possible
 #'   names of list elements are \code{"theta"}, \code{"beta"}, \code{"lambda"},
 #'   and \code{"weights"}, all of should be numerical vectors with starting
 #'   values. Default to \code{NULL}, which means that some relatively sensible
-#'   defaults are used.
+#'   defaults are used. Names of parameters must be given in all lower case.
 #'
 #' @param control Optional control object for the optimization procedure of
 #'   class \code{galamm_control} resulting from calling
@@ -172,9 +232,9 @@
 #'   data = mresp,
 #'   family = families,
 #'   family_mapping = family_mapping,
-#'   factor = list("level"),
+#'   factor = "level",
 #'   load.var = "itemgroup",
-#'   lambda = list(loading_matrix)
+#'   lambda = loading_matrix
 #' )
 #'
 #' # Summary information
@@ -214,8 +274,8 @@
 #'     (0 + loading | id),
 #'   data = dat,
 #'   load.var = "item",
-#'   lambda = list(loading_matrix),
-#'   factor = list("loading")
+#'   lambda = loading_matrix,
+#'   factor = "loading"
 #' )
 #'
 #' # We can plot the estimated smooth term
@@ -224,10 +284,10 @@
 #'
 #' # Interaction between observed and latent covariates ------------------------
 #' # Define the loading matrix
-#' lambda <- list(matrix(c(1, NA, NA), ncol = 1))
+#' lambda <- matrix(c(1, NA, NA), ncol = 1)
 #'
 #' # Define the regression functions, one for each row in the loading matrix
-#' factor_interactions <- list(list(~1, ~1, ~x))
+#' factor_interactions <- list(~1, ~1, ~x)
 #'
 #' # Fit the model
 #' mod <- galamm(
@@ -235,7 +295,7 @@
 #'   data = latent_covariates,
 #'   load.var = "type",
 #'   lambda = lambda,
-#'   factor = list("loading"),
+#'   factor = "loading",
 #'   factor_interactions = factor_interactions
 #' )
 #'
@@ -247,24 +307,84 @@
 #'
 #' @md
 galamm <- function(formula, weights = NULL, data, family = gaussian,
-                   family_mapping = rep(1L, nrow(data)),
+                   family_mapping = rep(1, nrow(data)),
                    load.var = NULL, lambda = NULL, factor = NULL,
                    factor_interactions = NULL,
+                   na.action = getOption("na.action"),
                    start = NULL, control = galamm_control()) {
-  data <- stats::na.omit(data)
+  if (!inherits(data, "data.frame")) {
+    stop("data must be a data.frame")
+  }
+
+  na.action <- match.arg(na.action, c("na.omit", "na.fail", "na.exclude"))
+  # Deal with potential missing values
+  if (any(vapply(data, function(x) any(is.infinite(x)), logical(1)))) {
+    stop("Infinite values in 'data'. galamm cannot handle this.")
+  }
+  if (any(vapply(data, function(x) any(is.nan(x)), logical(1)))) {
+    stop("NaN in 'data'. galamm cannot handle this.")
+  }
+  data <- eval(parse(text = paste0(na.action, "(data)")))
+
   if (nrow(data) == 0) stop("No data, nothing to do.")
+
   data <- as.data.frame(data)
   mc <- match.call()
 
-  family_list <- setup_family(family)
+  if (!methods::is(formula, "formula")) {
+    stop("formula must be a formula")
+  }
+  if (!is.null(weights) && !methods::is(weights, "formula")) {
+    stop("weights must be a formula")
+  }
   if (!is.vector(family_mapping)) {
     stop("family_mapping must be a vector.")
   }
-  if (is.numeric(family_mapping)) {
-    family_mapping <- as.integer(family_mapping)
+  if (nrow(data) != length(family_mapping)) {
+    stop("family_mapping must contain one index per row in data")
   }
 
-  stopifnot(length(family_list) == length(unique(family_mapping)))
+  family_list <- setup_family(family)
+
+  family_mapping <- as.integer(family_mapping)
+  if (length(family_list) != length(unique(family_mapping))) {
+    stop(
+      "family_mapping must contain a unique index for each element ",
+      "in family_list."
+    )
+  }
+
+  if (!is.null(load.var) && (length(load.var) > 1 || !is.character(load.var))) {
+    stop("load.var must be NULL or a character of length one")
+  }
+  if (!is.null(factor) && !is.character(factor)) {
+    stop("factor must be NULL or a character vector")
+  }
+  if (!is.null(lambda) && !is.numeric(lambda)) {
+    stop("lambda must either be NULL or a matrix or numeric vector")
+  }
+  if (!is.null(lambda) && !is.matrix(lambda)) {
+    lambda <- matrix(lambda, ncol = 1)
+    message("lambda converted to matrix with one column")
+  }
+  if (!is.null(lambda)) {
+    if (any(is.nan(lambda)) || any(is.infinite(lambda))) {
+      stop("elements of lambda can be either 0, 1, or NA")
+    }
+    if (!all(lambda[!is.na(lambda)] %in% c(0, 1))) {
+      stop("all non-NA values in lambda must be either 0 or 1")
+    }
+  }
+  if (any(vapply(list(load.var, lambda, factor), is.null, logical(1))) &&
+    any(vapply(
+      list(load.var, lambda, factor),
+      function(x) !is.null(x), logical(1)
+    ))) {
+    stop(
+      "load.var, lambda, and factor must either all have values or ",
+      "all be NULL."
+    )
+  }
 
   tmp <- setup_factor(load.var, lambda, factor, data)
   data <- tmp$data
@@ -280,6 +400,15 @@ galamm <- function(formula, weights = NULL, data, family = gaussian,
 
   response_obj <-
     setup_response_object(family_list, family_mapping, data, gobj)
+
+  check_matrix <- as.matrix(cbind(response_obj[, 1], gobj$lmod$X))
+  rank_check_covs <- qr(check_matrix[, -1, drop = FALSE])$rank
+  rank_check_response <- qr(check_matrix)$rank
+
+  if (rank_check_covs == rank_check_response) {
+    stop("Deterministic relationship between response and fixed effects.")
+  }
+
   lambda_mappings <- define_factor_mappings(
     gobj, load.var, lambda_orig, factor, factor_interactions, data
   )
@@ -289,7 +418,7 @@ galamm <- function(formula, weights = NULL, data, family = gaussian,
   theta_mapping <- gobj$lmod$reTrms$Lind - 1L
   theta_inds <- seq_along(gobj$lmod$reTrms$theta)
   beta_inds <- max(theta_inds) + seq_along(colnames(gobj$lmod$X))
-  lambda_inds <- max(beta_inds) + seq_along(lambda[[1]][lambda[[1]] >= 2])
+  lambda_inds <- max(beta_inds) + seq_along(lambda[lambda >= 2])
 
   if (!is.null(weights)) {
     weights_obj <- lme4::mkReTrms(lme4::findbars(weights), fr = data)
@@ -336,7 +465,7 @@ galamm <- function(formula, weights = NULL, data, family = gaussian,
       theta_mapping = theta_mapping,
       u_init = u_init,
       lambda = par[lambda_inds],
-      lambda_mapping_X = lambda_mappings$lambda_mapping_X,
+      lambda_mapping_X = as.integer(lambda_mappings$lambda_mapping_X),
       lambda_mapping_Zt = lambda_mappings$lambda_mapping_Zt,
       lambda_mapping_Zt_covs = lambda_mappings$lambda_mapping_Zt_covs,
       weights = par[weights_inds],
@@ -345,7 +474,7 @@ galamm <- function(formula, weights = NULL, data, family = gaussian,
       family_mapping = family_mapping - 1L,
       k = k,
       maxit_conditional_modes = maxit_conditional_modes,
-      lossvalue_tol = control$pwirls_tol_abs,
+      lossvalue_tol = control$pirls_tol_abs,
       gradient = gradient,
       hessian = hessian,
       reduced_hessian = control$reduced_hessian
@@ -464,6 +593,7 @@ galamm <- function(formula, weights = NULL, data, family = gaussian,
         logical(1)
       )),
     family = family_list,
+    family_mapping = family_mapping,
     factor_interactions = factor_interactions,
     fit = fit,
     fit_population = fit_population,
@@ -480,11 +610,11 @@ galamm <- function(formula, weights = NULL, data, family = gaussian,
   # Distinguish lambda indicies from regression coefficients in interactions
   # between observed and latent covariates
   if (length(lambda_inds) > 0) {
-    lambda_standard_inds <- intersect(lambda[[1]], lambda_orig[[1]]) - 1L
+    lambda_standard_inds <- intersect(lambda, lambda_orig) - 1L
     lambda_standard_inds <- lambda_standard_inds[lambda_standard_inds > 0] +
       min(lambda_inds) - 1
 
-    lambda_interaction_inds <- setdiff(lambda[[1]], lambda_orig[[1]]) - 1L
+    lambda_interaction_inds <- setdiff(lambda, lambda_orig) - 1L
     lambda_interaction_inds <-
       lambda_interaction_inds[lambda_interaction_inds > 0] +
       min(lambda_inds) - 1
